# cython: language_level=3
#  Drakkar-Software Async-Channel
#  Copyright (c) Drakkar-Software, All rights reserved.
#
#  This library is free software; you can redistribute it and/or
#  modify it under the terms of the GNU Lesser General Public
#  License as published by the Free Software Foundation; either
#  version 3.0 of the License, or (at your option) any later version.
#
#  This library is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
#  Lesser General Public License for more details.
#
#  You should have received a copy of the GNU Lesser General Public
#  License along with this library.

from async_channel cimport producer
from async_channel cimport consumer

from async_channel.producer cimport (
    Producer,
)
from async_channel.consumer cimport (
    Consumer,
    InternalConsumer,
    SupervisedConsumer,
)

__all__ = [
    "Producer",
    "Consumer",
    "InternalConsumer",
    "SupervisedConsumer",
]



# cython: language_level=3, wraparound=False
#  Drakkar-Software Async-Channel
#  Copyright (c) Drakkar-Software, All rights reserved.
#
#  This library is free software; you can redistribute it and/or
#  modify it under the terms of the GNU Lesser General Public
#  License as published by the Free Software Foundation; either
#  version 3.0 of the License, or (at your option) any later version.
#
#  This library is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
#  Lesser General Public License for more details.
#
#  You should have received a copy of the GNU Lesser General Public
#  License along with this library.
cimport async_channel.producer as producer

cdef class Channel(object):
    cdef public object logger

    cdef public str chan_id

    cdef public list producers
    cdef public list consumers

    cdef public producer.Producer internal_producer

    cdef public bint is_paused
    cdef public bint is_synchronized

    cpdef void add_new_consumer(self, object consumer, dict consumer_filters)
    cpdef list get_consumers(self)
    cpdef object get_producers(self)
    cpdef void unregister_producer(self, producer.Producer producer)
    cpdef list get_consumer_from_filters(self, dict consumer_filters)
    cpdef void flush(self)

    cdef list _filter_consumers(self, dict consumer_filters)
    cdef bint _should_pause_producers(self)
    cdef bint _should_resume_producers(self)

cpdef Channel set_chan(Channel chan, str name)
cpdef void del_chan(str name)
cpdef Channel get_chan(str chan_name)

cdef bint _check_filters(dict consumer_filters, dict expected_filters)



#  Drakkar-Software Async-Channel
#  Copyright (c) Drakkar-Software, All rights reserved.
#
#  This library is free software; you can redistribute it and/or
#  modify it under the terms of the GNU Lesser General Public
#  License as published by the Free Software Foundation; either
#  version 3.0 of the License, or (at your option) any later version.
#
#  This library is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
#  Lesser General Public License for more details.
#
#  You should have received a copy of the GNU Lesser General Public
#  License along with this library.
"""
Define async_channel logger implementation
"""
import logging


# pylint: disable=no-member, import-outside-toplevel
def get_logger(name: str = "") -> logging.Logger:
    """
    :param name: the logger name
    :return: the logger implementation, can be octobot_commons one or default python logging
    """
    try:
        import octobot_commons.logging as common_logging

        return common_logging.get_logger(logger_name=name)
    except ImportError:
        return logging.getLogger(name)



#  Drakkar-Software Async-Channel
#  Copyright (c) Drakkar-Software, All rights reserved.
#
#  This library is free software; you can redistribute it and/or
#  modify it under the terms of the GNU Lesser General Public
#  License as published by the Free Software Foundation; either
#  version 3.0 of the License, or (at your option) any later version.
#
#  This library is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
#  Lesser General Public License for more details.
#
#  You should have received a copy of the GNU Lesser General Public
#  License along with this library.
"""
Define async_channel project
"""

from async_channel import constants
from async_channel.constants import (
    CHANNEL_WILDCARD,
    DEFAULT_QUEUE_SIZE,
)

from async_channel import enums
from async_channel.enums import ChannelConsumerPriorityLevels

from async_channel import producer
from async_channel.producer import Producer

from async_channel import consumer
from async_channel.consumer import (
    Consumer,
    InternalConsumer,
    SupervisedConsumer,
)

PROJECT_NAME = "async-channel"
VERSION = "2.0.13"  # major.minor.revision

__all__ = [
    "CHANNEL_WILDCARD",
    "DEFAULT_QUEUE_SIZE",
    "ChannelConsumerPriorityLevels",
    "Producer",
    "Consumer",
    "InternalConsumer",
    "SupervisedConsumer",
    "PROJECT_NAME",
    "VERSION",
]



#  Drakkar-Software Async-Channel
#  Copyright (c) Drakkar-Software, All rights reserved.
#
#  This library is free software; you can redistribute it and/or
#  modify it under the terms of the GNU Lesser General Public
#  License as published by the Free Software Foundation; either
#  version 3.0 of the License, or (at your option) any later version.
#
#  This library is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
#  Lesser General Public License for more details.
#
#  You should have received a copy of the GNU Lesser General Public
#  License along with this library.
"""
Define async_channel implementation and usage
"""
from async_channel.channels import channel_instances
from async_channel.channels import channel

from async_channel.channels.channel_instances import (
    ChannelInstances,
    set_chan_at_id,
    get_channels,
    del_channel_container,
    get_chan_at_id,
    del_chan_at_id,
)
from async_channel.channels.channel import (
    Channel,
    set_chan,
    del_chan,
    get_chan,
)

__all__ = [
    "ChannelInstances",
    "set_chan_at_id",
    "get_channels",
    "del_channel_container",
    "get_chan_at_id",
    "del_chan_at_id",
    "Channel",
    "set_chan",
    "del_chan",
    "get_chan",
]
